"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const jest_mock_server_1 = require("jest-mock-server");
const file_temp_1 = require("../../libs/file-temp");
const testing_1 = require("../../testing");
const get_1 = require("./get");
const server = new jest_mock_server_1.MockServer();
let request;
beforeAll(async () => await server.start());
afterAll(async () => await server.stop());
beforeEach(async () => {
    await testing_1.Testing.reset();
    server.reset();
});
afterEach(async () => {
    await request.dispose();
});
test('send a "get" request then get json data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    request = await testing_1.Testing.newElement(get_1.Get, {
        url: server.getURL().toString() + '?time=' + Date.now().toString(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'json',
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
    expect(testing_1.Testing.vars.status).toBe(200);
    expect(testing_1.Testing.vars.statusText).toBe('OK MEN');
    expect(testing_1.Testing.vars.data).toEqual(bodyData);
});
test('send a "get" request then get blob data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    request = await testing_1.Testing.newElement(get_1.Get, {
        url: server.getURL().toString() + '?time=' + Date.now().toString(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'blob',
        vars: 'data'
    });
    await request.exec();
    expect(testing_1.Testing.vars.data).toBeInstanceOf(Blob);
});
test('send a "get" request then get arraybuffer data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        ctx.set('custom', 'thanh');
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    request = await testing_1.Testing.newElement(get_1.Get, {
        url: server.getURL().toString() + '?time=' + Date.now().toString(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'buffer',
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}',
            custom: '${this.response.headers?.custom}'
        }
    });
    await request.exec();
    expect(testing_1.Testing.vars.status).toBe(200);
    expect(testing_1.Testing.vars.statusText).toBe('OK MEN');
    expect(testing_1.Testing.vars.custom).toBe('thanh');
    expect(testing_1.Testing.vars.data).toBeInstanceOf(ArrayBuffer);
});
test('send a "get" request then ignore response data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
    });
    request = await testing_1.Testing.newElement(get_1.Get, {
        url: server.getURL().toString(),
        responseType: 'none',
        vars: 'data'
    });
    await request.exec();
    expect(testing_1.Testing.vars.data).toBeUndefined();
});
test('send a "get" request then get text data', async () => {
    const bodyData = {
        name: 'thanh',
        sex: 'male'
    };
    server.get('/')
        .mockImplementationOnce((ctx) => {
        ctx.status = 200;
        ctx.message = 'OK MEN';
        ctx.body = bodyData;
        expect(ctx.query.time).toBeDefined();
        expect(ctx.query.rd).toBeDefined();
    });
    request = await testing_1.Testing.newElement(get_1.Get, {
        url: server.getURL().toString() + '?time=' + Date.now().toString(),
        query: {
            rd: Math.random().toString()
        },
        responseType: 'text',
        vars: {
            status: '${this.response.status}',
            statusText: '${this.response.statusText}',
            data: '${this.response.data}'
        }
    });
    await request.exec();
    expect(testing_1.Testing.vars.status).toBe(200);
    expect(testing_1.Testing.vars.statusText).toBe('OK MEN');
    expect(typeof testing_1.Testing.vars.data).toBe('string');
});
test('should download a file', async () => {
    const tmp = new file_temp_1.FileTemp();
    try {
        const bodyData = 'thanh';
        server.get('/')
            .mockImplementationOnce((ctx) => {
            ctx.status = 200;
            ctx.message = 'OK MEN';
            ctx.body = bodyData;
        });
        request = await testing_1.Testing.newElement(get_1.Get, {
            url: server.getURL().toString(),
            saveTo: tmp.file
        });
        await request.exec();
        const fileContent = (0, fs_1.readFileSync)(tmp.file).toString();
        expect(fileContent).toBe(bodyData);
    }
    finally {
        tmp.remove();
    }
});
//# sourceMappingURL=get.spec.js.map