"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.LoggerLevel = void 0;
const chalk_1 = __importDefault(require("chalk"));
var LoggerLevel;
(function (LoggerLevel) {
    LoggerLevel["ALL"] = "trace";
    LoggerLevel["TRACE"] = "trace";
    LoggerLevel["DEBUG"] = "debug";
    LoggerLevel["INFO"] = "info";
    LoggerLevel["WARN"] = "warn";
    LoggerLevel["ERROR"] = "error";
    LoggerLevel["FATAL"] = "fatal";
    LoggerLevel["SILENT"] = "silent";
})(LoggerLevel = exports.LoggerLevel || (exports.LoggerLevel = {}));
const Mapping = {
    all: 0,
    trace: 2,
    debug: 4,
    info: 6,
    warn: 8,
    error: 10,
    fatal: 11,
    silent: 12
};
const Icon = {
    info: chalk_1.default.green('✔'),
    warn: chalk_1.default.yellow('⚠️'),
    error: chalk_1.default.red('✘'),
    fatal: chalk_1.default.red('❌'),
    trace: chalk_1.default.magenta('˖'),
    debug: '',
    others: '╎ ',
    debugBlock: {
        begin: chalk_1.default.red.dim('┌'),
        end: chalk_1.default.red.dim('└')
    }
};
const MappingValue = Object.keys(Mapping).reduce((sum, e) => {
    const lvNum = Mapping[e];
    sum[lvNum] = e;
    return sum;
}, {});
class Logger {
    context;
    indent;
    level = Mapping.all;
    indentString = '';
    get levelName() {
        return MappingValue[this.level];
    }
    constructor(level, context = '', indent = 0) {
        this.context = context;
        this.indent = indent;
        this.setLevel(level);
        if (this.indent)
            this.updateIndent(this.indent);
    }
    is(level) {
        const lv = typeof level === 'number' ? level : Mapping[level];
        return lv !== Mapping.silent ? (this.level <= lv) : (this.level === lv);
    }
    clone(context) {
        return new Logger(this.level, context || this.context, this.indent);
    }
    updateIndent(indent) {
        this.indent = indent;
        this.indentString = this.getIndentString(this.indent);
    }
    addIndent(indent = 1) {
        this.updateIndent(this.indent + indent);
    }
    removeIndent(indent = 1) {
        this.updateIndent(this.indent - indent);
    }
    getIndentString(indent) {
        const str = new Array(indent).fill(Icon.others).join('');
        return str && chalk_1.default.gray.dim(str);
    }
    formatWithoutIndent(msg, level) {
        const sp = msg ? ' ' : '';
        switch (level) {
            case LoggerLevel.INFO:
                return `${Icon.info}${sp}${msg}`;
            case LoggerLevel.WARN:
                return `${Icon.warn}${sp}${msg}`;
            case LoggerLevel.ERROR:
                return `${Icon.error}${sp}${msg}`;
            case LoggerLevel.FATAL:
                return `${Icon.fatal}${sp}${msg}`;
            case LoggerLevel.TRACE:
                return `${Icon.trace}${sp}${chalk_1.default.magenta(msg)}`;
            case LoggerLevel.DEBUG:
                return `${Icon.debug}${chalk_1.default.gray(`${msg}`)}`;
        }
        return `${msg}`;
    }
    format(msg, level) {
        return `${this.indentString}${this.formatWithoutIndent(msg, level)}`;
    }
    setLevel(level) {
        const lv = typeof level === 'number' ? level : Mapping[level];
        if (lv !== null && lv !== undefined)
            this.level = lv;
    }
    print(...args) {
        console.log(...args);
    }
    debugBlock(isStart, msg = this.context || '', ...prms) {
        if (!this.is(LoggerLevel.DEBUG))
            return;
        if (isStart) {
            console.log(`${this.indentString}%s %s`, Icon.debugBlock.begin, chalk_1.default.bgRed.dim('▾ ' + msg), ...prms);
            this.addIndent();
        }
        else {
            this.removeIndent();
            console.log(`${this.indentString}%s`, Icon.debugBlock.end);
        }
    }
    log(msg, ...prms) {
        if (!this.is(LoggerLevel.SILENT)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg) => {
                    console.log(`${this.indentString}${msg}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.indentString}${msg}`, ...prms);
            }
        }
    }
    info(msg, ...prms) {
        if (this.is(LoggerLevel.INFO)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg, i) => {
                    msg = (i === 0 ? '' : '  ') + msg;
                    console.log(`${this.format(msg, i !== 0 ? undefined : LoggerLevel.INFO)}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.format(msg, LoggerLevel.INFO)}`, ...prms);
            }
        }
    }
    debug(msg, ...prms) {
        if (this.is(LoggerLevel.DEBUG)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg, i) => {
                    msg = (i === 0 ? '' : '  ') + msg;
                    console.log(`${this.format(msg, i !== 0 ? undefined : LoggerLevel.DEBUG)}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.format(msg, LoggerLevel.DEBUG)}`, ...prms);
            }
        }
    }
    warn(msg, ...prms) {
        if (this.is(LoggerLevel.WARN)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg, i) => {
                    msg = (i === 0 ? '' : '  ') + msg;
                    console.log(`${this.format(msg, i !== 0 ? undefined : LoggerLevel.WARN)}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.format(msg, LoggerLevel.WARN)}`, ...prms);
            }
        }
    }
    trace(msg, ...prms) {
        if (this.is(LoggerLevel.TRACE)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg, i) => {
                    msg = (i === 0 ? '' : '  ') + msg;
                    console.log(`${this.format(msg, i !== 0 ? undefined : LoggerLevel.TRACE)}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.format(msg, LoggerLevel.TRACE)}`, ...prms);
            }
        }
    }
    error(msg, ...prms) {
        if (this.is(LoggerLevel.ERROR)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg, i) => {
                    msg = (i === 0 ? '' : '  ') + msg;
                    console.log(`${this.format(msg, i !== 0 ? undefined : LoggerLevel.ERROR)}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.format(msg, LoggerLevel.ERROR)}`, ...prms);
            }
        }
    }
    fatal(msg, ...prms) {
        if (this.is(LoggerLevel.ERROR)) {
            if (typeof msg === 'string') {
                msg.split('\n').forEach((msg, i) => {
                    msg = (i === 0 ? '' : '  ') + msg;
                    console.log(`${this.format(msg, i !== 0 ? undefined : LoggerLevel.FATAL)}`, ...prms);
                });
            }
            else {
                prms.splice(0, 0, msg);
                msg = '%j';
                console.log(`${this.format(msg, LoggerLevel.FATAL)}`, ...prms);
            }
        }
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map